#ifndef RSQLITE_COLUMNDATATYPE_H
#define RSQLITE_COLUMNDATATYPE_H

enum DATA_TYPE {
  DT_UNKNOWN,
  DT_BOOL,
  DT_INT,
  DT_INT64,
  DT_REAL,
  DT_STRING,
  DT_BLOB,
  DT_DATE,
  DT_DATETIME,
  DT_DATETIMETZ,
  DT_TIME
};

#endif  // RSQLITE_COLUMNDATATYPE_H
