#' RSQLite version
#'
#' Return the version of RSQLite.
#'
#' @return A character vector containing header and library versions of
#'   RSQLite.
#' @export
#' @examples
#' RSQLite::rsqliteVersion()
rsqliteVersion <- rsqliteVersion
