# Generated by cpp11: do not edit by hand

connection_connect <- function(path, allow_ext, flags, vfs, with_alt_types) {
  .Call(`_RSQLite_connection_connect`, path, allow_ext, flags, vfs, with_alt_types)
}

connection_valid <- function(con_) {
  .Call(`_RSQLite_connection_valid`, con_)
}

connection_release <- function(con_) {
  invisible(.Call(`_RSQLite_connection_release`, con_))
}

connection_in_transaction <- function(con_) {
  .Call(`_RSQLite_connection_in_transaction`, con_)
}

connection_copy_database <- function(from, to) {
  invisible(.Call(`_RSQLite_connection_copy_database`, from, to))
}

connection_import_file <- function(con, name, value, sep, eol, skip) {
  .Call(`_RSQLite_connection_import_file`, con, name, value, sep, eol, skip)
}

set_busy_handler <- function(con, r_callback) {
  invisible(.Call(`_RSQLite_set_busy_handler`, con, r_callback))
}

extension_load <- function(con, file, entry_point) {
  invisible(.Call(`_RSQLite_extension_load`, con, file, entry_point))
}

result_create <- function(con, sql) {
  .Call(`_RSQLite_result_create`, con, sql)
}

result_release <- function(res) {
  invisible(.Call(`_RSQLite_result_release`, res))
}

result_valid <- function(res_) {
  .Call(`_RSQLite_result_valid`, res_)
}

result_fetch <- function(res, n) {
  .Call(`_RSQLite_result_fetch`, res, n)
}

result_bind <- function(res, params) {
  invisible(.Call(`_RSQLite_result_bind`, res, params))
}

result_has_completed <- function(res) {
  .Call(`_RSQLite_result_has_completed`, res)
}

result_rows_fetched <- function(res) {
  .Call(`_RSQLite_result_rows_fetched`, res)
}

result_rows_affected <- function(res) {
  .Call(`_RSQLite_result_rows_affected`, res)
}

result_column_info <- function(res) {
  .Call(`_RSQLite_result_column_info`, res)
}

result_get_placeholder_names <- function(res) {
  .Call(`_RSQLite_result_get_placeholder_names`, res)
}

rsqliteVersion <- function() {
  .Call(`_RSQLite_rsqliteVersion`)
}
